@@ -93,12 +93,14 @@ key_type_to_gnutls_type (enum keyfile_type type)
    confused with actual gnutls functions -- such as the gnutls_read
    preprocessor macro.  */
 
+static bool ssl_initialized = false;
+
 static gnutls_certificate_credentials_t credentials;
 bool
 ssl_init (void)
 {
+  fprintf(stderr,"SSL_INIT\n");
   /* Becomes true if GnuTLS is initialized. */
-  static bool ssl_initialized = false;
   const char *ca_directory;
   DIR *dir;
   int ncerts = -1;
@@ -235,10 +237,17 @@ cert to be of the same type.\n"));
 void
 ssl_cleanup (void)
 {
+  fprintf(stderr,"SSL_CLEANUP\n");
+
+  if (!ssl_initialized)
+    return;
+
   if (credentials)
     gnutls_certificate_free_credentials(credentials);
 
   gnutls_global_deinit();
+
+  ssl_initialized = false;
 }
 
 struct wgnutls_transport_context
