@@ -1,5 +1,5 @@
 /* Collect URLs from HTML source.
-   Copyright (C) 1998-2012, 2015, 2018-2020 Free Software Foundation,
+   Copyright (C) 1998-2012, 2015, 2018-2021 Free Software Foundation,
    Inc.
 
 This file is part of GNU Wget.
@@ -529,8 +529,8 @@ tag_handle_link (int tagid _GL_UNUSED, struct taginfo *tag, struct map_context *
   /* All <link href="..."> link references are external, except those
      known not to be, such as style sheet and shortcut icon:
 
-     <link rel="stylesheet" href="...">
-     <link rel="shortcut icon" href="...">
+     <link rel="stylesheet" href="..."> or <link rel="alternate stylesheet" href="...">
+     <link rel="shortcut icon" href="..."> or <link rel="icon" href="...">
   */
   if (href)
     {
@@ -541,12 +541,16 @@ tag_handle_link (int tagid _GL_UNUSED, struct taginfo *tag, struct map_context *
           char *rel = find_attr (tag, "rel", NULL);
           if (rel)
             {
-              if (0 == c_strcasecmp (rel, "stylesheet"))
+              if (0 == c_strcasecmp (rel, "stylesheet") || 0 == c_strcasecmp (rel, "alternate stylesheet"))
                 {
                   up->link_inline_p = 1;
                   up->link_expect_css = 1;
                 }
-              else if (0 == c_strcasecmp (rel, "shortcut icon"))
+              else if (0 == c_strcasecmp (rel, "shortcut icon") || 0 == c_strcasecmp (rel, "icon"))
+                {
+                  up->link_inline_p = 1;
+                }
+              else if (0 == c_strcasecmp (rel, "manifest"))
                 {
                   up->link_inline_p = 1;
                 }
@@ -588,16 +592,16 @@ tag_handle_meta (int tagid _GL_UNUSED, struct taginfo *tag, struct map_context *
 
       struct urlpos *entry;
       int attrind;
-      int timeout = 0;
+      int timeout;
       char *p;
 
       char *refresh = find_attr (tag, "content", &attrind);
       if (!refresh)
         return;
 
-      for (p = refresh; c_isdigit (*p); p++)
-        timeout = 10 * timeout + *p - '0';
-      if (*p++ != ';')
+      timeout = strtol(refresh, &p, 10);
+
+      if (timeout < 0 || *p++ != ';')
         return;
 
       while (c_isspace (*p))
@@ -837,10 +841,7 @@ get_urls_html_fm (const char *file, const struct file_memory *fm,
 #endif
   xfree (meta_charset);
 
-  if (ctx.nofollow) {
-      logprintf(LOG_VERBOSE, _("no-follow attribute found in %s. Will not follow any links on this page\n"), file);
-  }
-  DEBUGP (("no-follow in %s: %d\n", file, ctx.nofollow));
+  DEBUGP (("nofollow in %s: %d\n", file, ctx.nofollow));
 
   if (meta_disallow_follow)
     *meta_disallow_follow = ctx.nofollow;
