@@ -37,7 +37,7 @@ extern int numurls;
 
 /* These global vars should be made static to retr.c and exported via
    functions! */
-extern SUM_SIZE_INT total_downloaded_bytes;
+extern wgint total_downloaded_bytes;
 extern double total_download_time;
 extern FILE *output_stream;
 extern bool output_stream_regular;
