@@ -1,16 +1,15 @@
 #!/usr/bin/env python3
 from sys import exit
 from test.http_test import HTTPTest
-from test.base_test import HTTP, HTTPS
+from test.base_test import HTTPS, SKIP_TEST
 from misc.wget_file import WgetFile
 import os
 
 """
     This test ensures that Wget can download files from HTTPS Servers
 """
-TEST_NAME = "HTTPS Downloads"
 if os.getenv('SSL_TESTS') is None:
-    exit (77)
+    exit (SKIP_TEST)
 
 ############# File Definitions ###############################################
 File1 = "Would you like some Tea?"
@@ -48,7 +47,6 @@ post_test = {
 }
 
 err = HTTPTest (
-                name=TEST_NAME,
                 pre_hook=pre_test,
                 test_params=test_options,
                 post_hook=post_test,
